;;; hanja.el --- KSC5601     ˻ Լ

;; Copyright (C) 1995, 1996 ϴϸ .

;;;==================================================================
;;;  ˻

(defvar hanja-map nil)
(defvar hanja-index 0)
(defvar hanja-vector nil)
(defvar hanja-wanted 0)
(defvar hanja-append-mode nil)
(defvar hanja-buffer-lock nil)
(defvar hanja-hangul-mode nil)
(defvar old-buffer nil)

(global-set-key "\C-cs" 'split-hanja-window)
(global-set-key "\C-cd" 'delete-hanja-window)
;(global-set-key [f9] 'split-hanja-window)
;(global-set-key [f8] 'delete-hanja-window)

(defun delete-hanja-window ()
  (interactive)
  (if hanja-buffer-lock
      (progn
	(set-buffer " *Hanja*")
	(setq hanja-wanted (buffer-substring (point) (+ (point) 2)))
	(save-excursion
	  (set-buffer old-buffer)
	  (if hanja-append-mode
	      (progn
		(if (not hanja-hangul-mode)
		    (goto-char (+ (point) 2))) ; ѱ    ̵
		(insert hanja-wanted))
	    (progn
	      (if hanja-hangul-mode
		  (goto-char (- (point) 2))) ; ѱ   ڷ ̵
	      (insert hanja-wanted)
	      (delete-region (point) (+ (point) 2))))
	  (delete-windows-on " *Hanja*")
	  (setq hanja-buffer-lock nil
		hanja-hangul-mode nil)
	  (let ((bufname (buffer-name old-buffer)))
	    (if (and (>= (length bufname) 10) 
		     (string=  (substring bufname 0 10) " *Minibuf-"))
		(switch-to-buffer-other-window old-buffer)
	      (switch-to-buffer old-buffer))))))
)

(defun cancel-hanja-window (message)
  (if hanja-buffer-lock
      (progn
	(delete-windows-on " *Hanja*")
	(set-buffer old-buffer)
	(setq hanja-buffer-lock nil
	      hanja-hangul-mode nil)
	(let ((bufname (buffer-name old-buffer)))
	  (if (and (>= (length bufname) 10) 
		   (string=  (substring bufname 0 10) " *Minibuf-"))
	      (switch-to-buffer-other-window old-buffer)
	    (switch-to-buffer old-buffer)))))
  (error message)
)

(defun split-hanja-window ()
  (interactive)
  (if hanja-buffer-lock
      (cancel-hanja-window "Hanja buffer was canceled..."))
  (let ((hankey nil)
        (len 0) (quot 0) (rem 1))
    (if (and (= han-kbd-state 1) 
	     (/= han-code-state 0)
	     (string= last-command "self-insert-command"))
	(setq hankey (buffer-substring (- (point) 2) (point))
	      hanja-hangul-mode t
	      han-code-state 0)
      (setq hankey (buffer-substring (point) (+ (point) 2))))
    (setq old-buffer (current-buffer))
    (save-excursion
      (set-buffer (get-buffer-create " *Hanja*"))
      (setq buffer-read-only nil
            hanja-vector nil
            hanja-buffer-lock t)
      (erase-buffer)
      (if (> (aref hankey 0) 128)
          (find-hanja-vector hankey)
	(cancel-hanja-window "Matching characters are not hangul..."))
      (if (not hanja-vector)
	  (cancel-hanja-window "Matching hanja does not exist..."))
      (setq len (length hanja-vector) 
            quot (/ len 10) rem (mod len 10))
      
      ;;  ͸ 쿡 
      (let ((start 0) (end  10) (word nil))
        (while (> quot 0)
          (setq word (substring hanja-vector start end))
          (insert (concat "\n " word))
          (setq start (+ start 10) end (+ end 10) quot (- quot 1)))
        (if (= rem 0)
            nil
          (setq word (substring hanja-vector start))
          (insert (concat "\n " word))))
      
      (setq buffer-read-only t)
      ;(goto-char (+ (point-min) 2))
      (switch-to-buffer-other-window " *Hanja*")
      (beginning-of-buffer)
      (forward-char 2)))
)

;; binary search ̿  ˻
(defun find-hanja-vector (hankey)
  (let* ((upper-bound 473)
         (hanja-index 236)
         (lower-bound   0)
         (hanja-key   nil)
         (hanja-vec   nil)
         (find-exaustive nil))
    (while (not (or hanja-vec find-exaustive))
      (setq hanja-key (car (aref hanja-map hanja-index)))
      (if (string= hankey hanja-key) ; ϴ  ã
          (setq hanja-vec (car (cdr (aref hanja-map hanja-index))))
        (if (= hanja-index lower-bound) ;     
            (setq find-exaustive t)
          (progn
            (if (string< hankey hanja-key)
                (setq upper-bound hanja-index)
              (setq lower-bound (+ hanja-index 1)))
            (let* ((bound-sum (+ upper-bound lower-bound)))
              (if (= 0 (mod bound-sum 2))
                  (setq hanja-index (/ bound-sum 2))
                (setq hanja-index (/ (- bound-sum 1) 2)))))))
      (if (or (>= hanja-index upper-bound)
              (< hanja-index lower-bound))
          (setq find-exaustive t)))
    (setq hanja-vector hanja-vec))
)

; sequential search ̿  ˻
; (defun find-hanja-vector (hankey)
;  (let* ((hanja-index 0)
;        (hanja-vec nil))
;    (while (and (< hanja-index 474) (not hanja-vec))
;      (if (string= hankey (car (aref hanja-map hanja-index)))
;         (setq hanja-vec (cdr (aref hanja-map hanja-index)))
;       (setq hanja-index (+ 1 hanja-index))))
;    (setq hanja-vector hanja-vec))
; )

;;;==================================================================
;;;  

(defun define-hanja-map (hankey hanja-vec)
  (if (not hanja-map)
      (setq hanja-map (make-vector 473 nil)))
  (aset hanja-map hanja-index (list hankey hanja-vec))
  (setq hanja-index (+ 1 hanja-index))
)

(define-hanja-map ""  "ʡʢʣʤʥʦʧʨʩʪʫʬʭʮʯʰʱʲʳʴʵʶʷʸʹʺʻʼʽ")
(define-hanja-map ""  "ʾʿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "ˡˢˣˤ˥˦")
(define-hanja-map ""  "˧˨˩˪˫ˬ˭ˮ˯˰˱˲˳˴˵˶˷˸˹˺˻˼˽˾")
(define-hanja-map ""  "˿")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "̡̢̣")

(define-hanja-map ""  "̤̥̦")
(define-hanja-map ""  "̧̨̩")
(define-hanja-map ""  "̪̫̬̭̮̯̰")
(define-hanja-map ""  "̴̵̶̷̸̱̲̳̹̺̻")
(define-hanja-map ""  "̼̽̾̿")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ͣͤͥͦͧͨͩͪͫͬͭͮ͢͡")

(define-hanja-map ""  "ͯͰͱͲͳʹ͵Ͷͷ͸͹ͺͻͼͽ;Ϳ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "Ρ΢ΣΤΥΦΧΨ")

(define-hanja-map ""  "ΩΪΫά")
(define-hanja-map ""  "έήίΰαβγδεζηθικλμν")
(define-hanja-map ""  "ξο")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ϡϢϣϤϥϦϧϨϩϪϫϬϭϮϯϰϱϲϳϴϵ϶ϷϸϹϺϻϼϽϾϿ")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "СТ")

(define-hanja-map ""  "УФХЦЧШЩЪЫЬЭЮЯаб")
(define-hanja-map ""  "вгдежзи")
(define-hanja-map ""  "й")
(define-hanja-map ""  "клмноп")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ѡѢѣѤѥѦѧѨѩѪѫѬѭѮѯѰѱѲѳѴѵѶѷѸѹѺѻѼѽѾѿ")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ҡҢңҤ")

(define-hanja-map ""  "ҥҦҧҨҩҪҫ")
(define-hanja-map ""  "ҬҭҮүҰұ")
(define-hanja-map ""  "Ҳ")
(define-hanja-map ""  "ҳ")
(define-hanja-map ""  "ҴҵҶ")
(define-hanja-map ""  "ҷҸҹҺ")
(define-hanja-map ""  "һҼ")
(define-hanja-map ""  "ҽҾҿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "ӡӢӣӤӥӦӧӨөӪӫӬӭӮӯӰӱӲӳӴ")
(define-hanja-map ""  "ӵӶӷӸӹ")
(define-hanja-map ""  "ӺӻӼӽӾӿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ԡԢԣԤԥԦԧԨԩԪԫԬԭԮԯ԰ԱԲԳԴԵԶԷ")

(define-hanja-map ""  "ԸԹԺԻԼԽԾԿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "աբգդ")

(define-hanja-map ""  "եզէըթժիլխ")
(define-hanja-map ""  "ծկհձղճմյն")
(define-hanja-map ""  "շո")
(define-hanja-map ""  "չպջռսվտ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "֢֣֤֥֦֧֡֨")

(define-hanja-map ""  "֪֭֮֩֫֬")
(define-hanja-map ""  "ְֱֲֳ֯")
(define-hanja-map ""  "ִ")
(define-hanja-map ""  "ֵֶַָֹֺֻּֽ־ֿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "סע")

(define-hanja-map ""  "ף")
(define-hanja-map ""  "פץצקרשת׫׬׭׮ׯװ")
(define-hanja-map ""  "ױײ׳״׵׶׷׸׹׺׻׼׽׾")
(define-hanja-map ""  "׿")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ءآأ")

(define-hanja-map ""  "ؤإئابةتث")
(define-hanja-map ""  "جحخدذر")
(define-hanja-map ""  "زسشصضطظعغػؼؽؾؿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "١")

(define-hanja-map ""  "٢٣٤٥٦٧٨٩٪٫٬٭ٮٯٰ")
(define-hanja-map ""  "ٱ")
(define-hanja-map ""  "ٲٳٴٵٶٷٸٹٺٻټٽپٿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ڡڢڣڤڥڦڧ")

(define-hanja-map ""  "ڨکڪ")
(define-hanja-map ""  "ګڬڭڮگڰڱڲڳڴڵڶڷڸڹںڻڼڽ")
(define-hanja-map ""  "ھڿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ۣۡۢۤۥۦ")

(define-hanja-map ""  "ۧۨ۩۪ۭ۫۬ۮۯ۰۱۲۳۴۵۶۷۸۹ۺۻۼ۽۾ۿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ܡܢܣܤ")

(define-hanja-map ""  "ܥܦܧܨܩܪܫ")
(define-hanja-map ""  "ܬܭܮܯ")
(define-hanja-map ""  "ܱܴܷܸܹܻܼܾܰܲܳܵܶܺܽܿ")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ݡݢݣݤݥݦݧݨݩݪݫݬݭݮݯݰݱݲݳݴݵݶݷݸݹݺݻݼݽݾݿ")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "ޡޢޣޤޥަާިީުޫެ")

(define-hanja-map ""  "ޭޮޯްޱ޲޳޴޵޶޷޸޹޺")
(define-hanja-map ""  "޻޼޽޾")
(define-hanja-map ""  "޿")
(define-hanja-map ""  "")

(define-hanja-map ""  "ߡߢߣߤߥߦߧߨߩߪ߫߬")
(define-hanja-map ""  "߭߮߯߰߱")
(define-hanja-map ""  "߲߳ߴߵ߶߷߸߹")
(define-hanja-map ""  "ߺ߻߼߽")
(define-hanja-map ""  "߾߿")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map "¡"  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map "â"  "")
(define-hanja-map "ä"  "")
(define-hanja-map "å"  "")
(define-hanja-map "ó"  "")
(define-hanja-map "ô"  "")
(define-hanja-map "õ"  "")
(define-hanja-map "ö"  "")
(define-hanja-map "÷"  "")
(define-hanja-map "ø"  "")
(define-hanja-map "û"  "")
(define-hanja-map "ü"  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map "ġ"  "")
(define-hanja-map "Ģ"  "")
(define-hanja-map "ģ"  "")
(define-hanja-map "ĥ"  "")
(define-hanja-map "ħ"  "")
(define-hanja-map "Ĩ"  "")
(define-hanja-map "Ī"  "")
(define-hanja-map ""  "")
(define-hanja-map "Ÿ"  "")
(define-hanja-map "Ź"  "")

(define-hanja-map "ź"  "")
(define-hanja-map "Ż"  "")
(define-hanja-map "Ž"  "")
(define-hanja-map "ž"  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map "Ư"  "")
(define-hanja-map "ƴ"  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map "ǥ"  "")

(define-hanja-map "ǰ"  "")
(define-hanja-map "ǳ"  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")

(define-hanja-map ""  "")
(define-hanja-map "ȣ"  "")
(define-hanja-map "Ȥ"  "")
(define-hanja-map "ȥ"  "")
(define-hanja-map "Ȧ"  "")
(define-hanja-map "ȫ"  "")
(define-hanja-map "ȭ"  "")
(define-hanja-map "Ȯ"  "")
(define-hanja-map "ȯ"  "")
(define-hanja-map "Ȱ"  "")
(define-hanja-map "Ȳ"  "")
(define-hanja-map "ȸ"  "")
(define-hanja-map "ȹ"  "")
(define-hanja-map "Ⱦ"  "")
(define-hanja-map "ȿ"  "")

(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")
(define-hanja-map ""  "")


